package de.vortexhq.lobby.Utils.Data;

import com.google.gson.JsonObject;
import de.vortexhq.lobby.Utils.ArrayHelper;
import de.vortexhq.lobby.interfaces.IDataHandler;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

import javax.annotation.Nullable;

public class MySQL implements IDataHandler {

    private Connection con;

    /**te
     * handles all mysql database operations
     * @param url the target url for the database
     * @param username the username for the database
     * @param password the password for the database
     * @param database the name of the datbase (not yet required)
     * @throws Exception if anything failes during connection attempt an exception throws
     */
    public MySQL(String url, String username, String password, String database) throws Exception{
        try {
            connect(url, username, password);
        } catch(Exception ex) {
            throw new Exception("could not connect to database due to error: " + ex.getLocalizedMessage());
        }
    }

    /**
     * connects to the Database
     * @param url target url to the database
     * @param username username for the database
     * @param password password for the database
     */
    private Connection connect(String url, String username, String password){
        try {
            con = DriverManager.getConnection(url, username, password);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * creates a table in the set database
     * @param name the tables name
     * @param fieldMap a map given with fieldnames and datatypes
     * @throws SQLException
     */
    @Override
    public void createTable(String name, Map<String,String> fieldMap) {
        String fields = "";

        fields += "(";
        for (Map.Entry<String, String> entry : fieldMap.entrySet()) {
            System.out.println(entry.getKey() + ":" + entry.getValue());
            fields += entry.getKey() + " " + entry.getValue();
        }
        fields += ")";

        try {
            this.con.createStatement().executeQuery("CREATE TABLE IF NOT EXISTS " + name + " " + fields);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /**
     * execute a given statement on a mysql database
     * this only returns a single given fields value for return of rows use getResults
     * @param stmt the statement used to query the database
     * @param target used to specify which field is required
     * @return if found values from the database of type int
     */
    @Override
    public int getInt(String stmt, String target) {
        try {
            int returning = 0;
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                returning = res.getInt(target);
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return 0;
    }

    /**
     * execute a given statement on a mysql database
     * this only returns a single given fields value for return of rows use getResults
     * @param stmt the statement used to query the database
     * @param target used to specify which field is required
     * @return if found values from the database of type int
     */
    @Override
    public int[] getInts(String stmt, String target) {
        try {
            int[] returning = {};
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                ArrayHelper.push(returning, res.getInt(target));
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @param a2 not used
     * @return if found values from the database of type int
     */
    @Override
    public double getDouble(String stmt, @Nullable String target) {
        try {
            double returning = 0;
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                returning = res.getDouble(target);
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return 0;
    }

    @Override
    public double[] getDoubles(String stmt, String target) {
        try {
            double[] returning = {};
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                ArrayHelper.push(returning, res.getDouble(target));
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @param target states what field is wanted
     * @return if found values from the database of type string
     */
    @Override
    public String getString(String stmt, String target) {
        try {
            String returning = "";
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                returning = res.getString(target);
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @param target not used
     * @return if found values from the database of type string as array
     */
    @Override
    public String[] getStrings(String stmt, String target) {
        try {
            String[] returning = {};
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                ArrayHelper.push(returning, res.getString(target));
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @param target not used
     * @return if found values from the database of type boolean
     */
    @Override
    public boolean getBool(String stmt, String target) {
        try {
            boolean returning = false;
            ResultSet res = this.con.createStatement().executeQuery(stmt);

            while (res.next()) {
                returning = res.getBoolean(target);
            }

            return returning;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @param target not used
     * @return if found values from the database of type boolean as array
     */
    @Override
    public boolean[] getBools(String stmt, String target) {
        boolean[] result = {};

        try {
            ResultSet rs = this.con.createStatement().executeQuery(stmt);
            while (rs.next()) {
                result = ArrayHelper.push(result,rs.getBoolean(target));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * execute a given statement on a mysql database
     * @param stmt the statement used to query the database
     * @return if found values from the database of type int
     */
    @Override
    public boolean writeData(String stmt) {
        return false;
    }


    /**
     * executes a query and returns the unproccessed resultset
     * @param a1 the query given as a string
     * @return a resultset to continue proccessing with
     */
    @Override
    public ResultSet getResults(String a1) {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Unimplemented method 'getResults'");
    }


    @Override
    public JsonObject getResultAsJSON(String a1) {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Unimplemented method 'getResultAsJSON'");
    }


    @Override
    public JsonObject[] getResultsAsJSON(String a1) {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Unimplemented method 'getResultsAsJSON'");
    }


    @Override
    public Map<Byte[], Byte[]> getResultsAsMap(String a1) {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Unimplemented method 'getResultsAsMap'");
    }
}
