package de.vortexhq.lobby.Utils;

import java.util.Map;

import de.vortexhq.lobby.Lobby;
import de.vortexhq.lobby.Utils.Data.MySQL;
import de.vortexhq.lobby.Utils.Data.SQLite;
import de.vortexhq.lobby.interfaces.IDataHandler;

public class Datahandler {
    private Lobby plugin;
    private IDataHandler handler;
    private String type;

    /**
     * handles all data operations
     * @param plugin a plugin reference
     * @param type the type of datahandling
     * @param url if mysql selected database url
     * @param username if mysql selected database username
     * @param password if mysql selected databaes password
     */
    public Datahandler(Lobby plugin, String type, String url, String username, String password) {
        plugin.Logger.Info("Initialize Database handler");
        this.plugin = plugin;
        this.type = type;

        switch (type) {
            case "mysql":
                try {
                    this.plugin.getLogger().info("initializing mysql");
                    this.handler = new MySQL(url, username, password, "");
                } catch (Exception e) {
                    this.plugin.getLogger().info("an error occured during mysql intialization exception thrown: " + e.getLocalizedMessage());
                    e.printStackTrace();
                    return;
                }
                break;
            case "sqlite":
                this.plugin.Logger.Info("Initialize SQLite Datasource");
                try {
                    this.handler = new SQLite(this.plugin, url);
                } catch (Exception e) {
                    e.printStackTrace();
                    return;
                }
                break;
        }
    }

    public void createTable(String tablename, Map<String, String> fieldmap){
        try {
            this.handler.createTable(tablename, fieldmap);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
    }

    public IDataHandler getDatahandler(){
        return handler;
    }
}
