package de.vortexhq.lobby.interfaces;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.Map;

import com.google.gson.JsonObject;
import netscape.javascript.JSObject;

public interface IDataHandler {
    /**
     * get integers values from a datasource
     * @param a1 first argument usually used as target value
     * @param a2 second argument usually used as value or extra file
     * @return value of type int
     */
    public int getInt(String a1, String a2);
    public int[] getInts(String a1, String a2);

    /**
     * get floats values from a datasource
     * @param a1 first argument usually used as target value
     * @param a2 second argument usually used as value or extra file
     * @return value of type float
     */
    public double getDouble(String a1, String a2);
    public double[] getDoubles(String a1, String a2);

    /**
     * get strings values from a datasource
     * @param a1 first argument usually used as target value
     * @param a2 second argument usually used as value or extra file
     * @return value of type String
     */
    public String getString(String a1, String a2);
    public String[] getStrings(String a1, String a2);

    /**
     * get boolean values from a datasource
     * @param a1 first argument usually used as target value
     * @param a2 second argument usually used as value or extra file
     * @return value of type int
     */
    public boolean getBool(String a1, String a2);
    public boolean[] getBools(String a1, String a2);

    /**
     * executes a query and returns the unproccessed resultset
     * @param a1 the query given as a string
     * @return a resultset to continue proccessing with
     */
    public ResultSet getResults(String a1);

    /**
     * executes given query and returns result as json object
     * @param a1 the query to execute
     * @return the result as JSONObject
     */
    public JsonObject getResultAsJSON(String a1);
    public JsonObject[] getResultsAsJSON(String a1);

    /**
     * executes a query and returns untyped results as map
     * @param a1 the query given as a string
     * @return a resultset to continue proccessing with
     */
    public Map<Byte[], Byte[]> getResultsAsMap(String a1);

    /**
     * write values to a set datasource
     * @param a1 first argument usually used as location or statement
     * @param a2 second argument usually used as value or config location
     * @param a3 third argument usually used as value or extra file
     * @return if the write operation was successful
     */
    public boolean writeData(String a1, Byte[] a3);

    /**
     * creates table if the datasource supports it
     * @param name the table name
     * @param fieldMap a map[string]string of the files and their types for that given table
     */
    public void createTable(String name, Map<String,String> fieldMap);
}
